/* ############################################################# *

bin2csv: convert a stars.bin file to a comma delimited text file
version 1.0

Written by: Vurt, <sbenoit@mnsi.net>
Last modified: 2000/01/15

usage 1: bin2csv <bin file> <csv file>
usage 2: bin2csv <csv file>
usage 3: bin2csv



Disclaimer:  Neither Particle Systems nor Infogrames are
responsible for this code, or anything it does to your
machine, and cannot be held accountable.  For that matter,
I'm not accountable for anything this code does either, etc.

Feel free to modify this code, and distribute it, but please
credit the original author.  If you make any really good
mods, or find and fix a bug, please send me a copy of what
you did so that I can incorporate it into the original
version.

Finally, you are prohibited from making any money from my work.


Special thanks go out to Particle Systems, for obvious reasons,
and to Nanoprobe for getting me started on this, and for his
help debugging.

 * ############################################################# */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_FILENAME_LENGTH 127
#define STAR_NAME_SIZE 20
#define DEFAULT_INFILENAME  "stars.bin"
#define DEFAULT_OUTFILENAME "stars.csv"

typedef struct StarData StarData;
struct StarData {
  float x;
  float y;
  float z;
  char name[STAR_NAME_SIZE+1];
  float apparent_mag;
  float absolute_mag;
  unsigned char system;
  unsigned long flags;
};

main(int argc, char *argv[]);
void clearstar(StarData *star);
int readstar(StarData *star, FILE *fp);
int writestar(StarData *star, FILE *fp);
void replace(char *p, char q, char r);

// it all starts with main...
main(int argc, char *argv[])
{
  StarData star;
  StarData *pstar = &star;

  FILE *fpi;
  FILE *fpo;
  char inputfile[MAX_FILENAME_LENGTH+1] = DEFAULT_INFILENAME;
  char outputfile[MAX_FILENAME_LENGTH+1] = DEFAULT_OUTFILENAME;
  int eof = 0;

  if (1 == argc) // no input, default files
  {
    ; // nothing to do
  }
  else if (2 == argc) // assume only output file specified
  {
	strncpy(outputfile, argv[1], MAX_FILENAME_LENGTH);
  }
  else if (3 == argc) // input and output files specified
  {
    strncpy(inputfile, argv[1], MAX_FILENAME_LENGTH);
    strncpy(outputfile, argv[2], MAX_FILENAME_LENGTH);
  }
  else // too many arguments
  {
    printf("bin2csv: too many arguments\n");
    exit(1);
  }

  strcat(inputfile, '\0');
  strcat(outputfile, '\0');

  // open the inputfile and make sure it's valid
  fpi = fopen(inputfile, "r+b");

  if (!fpi)
  {
	printf("bin2csv: unable to open %s\n", inputfile);
	exit(2);
  }

  // open the outputfile and make sure it's valid
  fpo = fopen(outputfile, "w");

  if (!fpo)
  {
	printf("bin2csv: unable to open %s\n", outputfile);
	exit(3);
  }

  printf("bin2csv: building %s...  ", outputfile);

  // while not the end of file, convert
  while (!eof)
  {
	clearstar(pstar);
	eof = readstar(pstar, fpi);
	if (!eof) writestar(pstar, fpo);
  }

  printf("finished!\n");

  // clean up
  fclose(fpi);
  fclose(fpo);
}


// initialize the star
void clearstar(StarData *star)
{
  star->x = 0.0;
  star->y = 0.0;
  star->z = 0.0;
  memset(star->name, '\0', STAR_NAME_SIZE+1);
  star->apparent_mag = 0.0;
  star->absolute_mag = 0.0;
  star->system = '\0';
  star->flags = 0;
}


// read the star from the specified file, return 1 if at the EOF
// warning: things may get ugly if the input file contains truncated records
int readstar(StarData *star, FILE *fp)
{
  int status;

  status = fread(&star->x, sizeof(float), 1, fp);

  if (status != 0)
  {
    fread(&star->y, sizeof(float), 1, fp);
    fread(&star->z, sizeof(float), 1, fp);
    fread(&star->name, STAR_NAME_SIZE, 1, fp);
    fread(&star->apparent_mag, sizeof(float), 1, fp);
    fread(&star->absolute_mag, sizeof(float), 1, fp);
    fread(&star->system, sizeof(char), 1, fp);
    fread(&star->flags, sizeof(long), 1, fp);

    // make sure star->name is terminated
    strcat(star->name, '\0');

    return 0;
  }
  else
  {
  	return 1;
  }
}


// write the star to the specified file
int writestar(StarData *star, FILE *fp)
{
  replace(star->name, ',', '*');

  fprintf(fp, "%f,%f,%f,%s,%f,%f,%i,%lu\n", star->x, star->y, star->z, star->name,
          star->apparent_mag, star->absolute_mag, star->system, star->flags);
  fflush(fp);

  return 0;
}


// replace any instance of q in the string p with r
void replace(char *p, char q, char r)
{
  int i;

  for (i = 0; i < strlen(p); i++)
    if (p[i] == q)
      p[i] = r;

}